﻿using System.Collections.Generic;
using TSLab.DataSource;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;

namespace TSLab.Community.Ichimoku2 {
  public abstract class BaseSen {
    public virtual int Period { get; set; }

    static double CalcMiddle(IReadOnlyList<IDataBar> values, int start, int finish) {
      double min = float.MaxValue, max = -float.MaxValue;
      for (int i = start; i <= finish; i++) {
        var v = values[ i ];
        if (v.High > max)
          max = v.High;
        if (v.Low < min)
          min = v.Low;
      }
      return 0.5 * (max + min);
    }

    static double CalcSmoothMiddle(IReadOnlyList<IDataBar> values, int start, int finish) {
      double m = 0;
      for (int i = start; i <= finish; i++) {
        var v = values[ i ];
        m += v.High + v.Low + v.Close + v.Open;
      }
      m /= 4 * (finish - start + 1);
      return m;
    }

    public IList<double> Execute(ISecurity source) {
      int c = source.Bars.Count;
      var list = new double[ c ];
      for (int i = Period-1; i < c; i++)
        //list[i] = CalcMiddle( source.Bars, i - Period + 1, i );
        list[ i ] = CalcSmoothMiddle( source.Bars, i - Period + 1, i );
      return list;
    }
  }

  /// <summary>
  /// Tenkan-sen - короткая линия тренда (дословный перевод - линия переворота).
  /// Линия трактуется как указатель направления тренда: чем больше крутизна этой линии,
  /// тем более ярко выражен тренд. Если линия горизонтальна - рынок вошел в канал.
  /// Кроме того, как и любая средняя, эта линия может
  /// рассматриваться как поддержка или сопротивление.
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class TenkanSen2 : BaseSen, IBar2DoubleHandler {
    [HandlerParameter(true, "9", Min = "3", Max = "25", Step = "1")]
    public override int Period { get; set; }
  }

  /// <summary>
  /// Kijun (Kijun-sen), "линия стандарта" - среднее значение цены за второй,
  /// более продолжительный промежуток времени.
  /// Используется как показатель движения рынка. Если цена выше нее, цены, 
  /// вероятно, будут продолжать расти. Когда цена пересекает эту линию,
  /// вероятно дальнейшее изменение тренда.
  /// Другим вариантом использования Kijun является подача сигналов. Сигнал к покупке
  /// генерируется когда линия Tenkan пересекает Kijun снизу вверх.
  /// Если сверху-вниз - сгнал к продаже.
  /// 
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class KijunSen2 : BaseSen, IBar2DoubleHandler {
    [HandlerParameter(true, "26", Min = "5", Max = "52", Step = "1")]
    public override int Period { get; set; }
  }

  /// <summary>
  /// SenkouA (Senkou Span A) - среднее от <see cref="TenkanSen2"/> и <see cref="KijunSen2"/>,
  /// сдвинутое вперед на величину периода KijunSen (26).
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class SenkouA2 : IBar2DoubleHandler {
    [HandlerParameter(true, "9", Min = "3", Max = "25", Step = "1")]
    public int TenkanPeriod { get; set; }

    [HandlerParameter(true, "26", Min = "5", Max = "52", Step = "1")]
    public int KijunPeriod { get; set; }

    public IList<double> Execute(ISecurity source) {
      var tenkan = new TenkanSen2 { Period = TenkanPeriod }.Execute(source);
      var kijun = new KijunSen2 { Period = KijunPeriod }.Execute(source);

      var count = source.Bars.Count;
      var result = new double[ count ];
      for (int i = KijunPeriod + KijunPeriod; i < count; i++)
        result[ i ] = 0.5 * (tenkan[ i - KijunPeriod ] + kijun[ i - KijunPeriod ]);
      return result;
    }
  }

  /// <summary>
  /// SenkouB (Senkou Span B) - среднее значение цены за третий временной интервал (52),
  /// сдвинутое вперед на величину второго интервала (26).
  /// Расстояние между линиями <see cref="SenkouA2"/> и <see cref="SenkouB2"/> штрихуется и 
  /// называется "облаком". Если цена находится между этими линиями, рынок считается нетрендовым,
  /// а края облака в таком случае образуют поддержку и сопротивление.
  /// Если цена находится над облаком, то обе линии - поддержки. Если под, то сопротивления.
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class SenkouB2 : IBar2DoubleHandler {
    [HandlerParameter(true, "52", Min = "25", Max = "255", Step = "1")]
    public int Period { get; set; }

    [HandlerParameter(true, "26", Min = "5", Max = "52", Step = "1")]
    public int KijunPeriod { get; set; }

    public IList<double> Execute(ISecurity source) {
      var avg = new TenkanSen2 { Period = Period }.Execute(source);

      var count = source.Bars.Count;
      var result = new double[ count ];
      for (int i = Period + KijunPeriod; i < count; i++)
        result[ i ] = avg[ i - KijunPeriod ];
      return result;
    }
  }

  /// <summary>
  /// Chinkou Span (Chinkou) - показывает цену закрытия текущей свечи, 
  /// сдвинутую назад на величину второго временного интервала (26).
  /// Рекомендуется использовать с осторожностью, поскольку индикатор "смотрит в будущее" 
  /// на <see cref="KijunPeriod"/> (26) баров.
  /// Если линия пересекает график цены снизу-вверх, это является сигналом к покупке.
  /// Если сверху-вниз - сигналом к продаже.
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class Chinkou2 : IBar2DoubleHandler {
    [HandlerParameter(true, "26", Min = "5", Max = "52", Step = "1")]
    public int KijunPeriod { get; set; }

    public IList<double> Execute(ISecurity source) {
      var bars = source.Bars;
      var count = bars.Count;

      var result = new double[ count ];
      for (int i = 0; i < count - KijunPeriod; i++)
        result[ i ] = bars[ i + KijunPeriod ].Close;
      return result;
    }
  }
  
  /// <summary>
  /// Только для визуализации!
  /// Индикатор облака между линиями <see cref="SenkouA2"/> и <see cref="SenkouB2"/>.
  /// Рисует ломаную линию от одной границы до другой.
  /// </summary>
  [HandlerCategory("Ichimoku")]
  public class IchimokuCloud : IBar2DoubleHandler {
    [HandlerParameter(true, "9", Min = "3", Max = "25", Step = "1")]
    public int TenkanPeriod { get; set; }
    [HandlerParameter(true, "26", Min = "5", Max = "52", Step = "1")]
    public int KijunPeriod { get; set; }
    [HandlerParameter(true, "52", Min = "25", Max = "255", Step = "1")]
    public int Period { get; set; }

    public IList<double> Execute(ISecurity source) {
      var senkouA = new SenkouA2 { KijunPeriod = KijunPeriod, TenkanPeriod = TenkanPeriod }.Execute(source);
      var senkouB = new SenkouB2 { Period = Period, KijunPeriod = KijunPeriod }.Execute(source);

      var barsCount = source.Bars.Count;
      var cloud = new double[ barsCount ];
      for (int i = 0; i < barsCount; i++)
        cloud[ i ] = (i & 1) == 1 ? senkouA[ i ] : senkouB[ i ];
      return cloud;
    }
  }
}